==> derby-0.1.0.tgz: package/lib/loader.js <==
var basename, crypto, cssCache, dirname, exists, findPath, fs, isProduction, join, jsCache, loadTemplates, nib, onlyWhitespace, parseHtml, racer, stylus, trim, views, _ref;
_ref = require('path'), dirname = _ref.dirname, basename = _ref.basename, join = _ref.join, exists = _ref.exists;
fs = require('fs');
crypto = require('crypto');
stylus = require('stylus');
nib = require('nib');
racer = require('racer');
trim = require('./View').trim;
parseHtml = require('./html').parse;
cssCache = {};
jsCache = {};
onlyWhitespace = /^[\s\n]*$/;
findPath = function(root, name, dir, extension, callback) {
  var path;
  if (name) {
    root = join(root, dir, name);
  }
  path = root + extension;
  return exists(path, function(value) {
    if (value) {
      return callback(path);
    }
    path = join(root, 'index' + extension);
    return exists(path, function(value) {
      return callback(value ? path : null);
    });
  });
};
loadTemplates = function(root, fileName, get, alias, templates, callback) {
  callback.count = (callback.count || 0) + 1;
  return findPath(root, fileName, 'views', '.html', function(path) {
    var got;
    if (!path) {
      if (fileName) {
        return callback({});
      } else {
        throw new Error("Can't find " + root + "/views/" + fileName);
      }
    }
    got = false;
    return fs.readFile(path, 'utf8', function(err, template) {
      var from, importName, name;
      if (err) {
        return callback(err);
      }
      name = '';
      from = '';
      importName = '';
      parseHtml(template, {
        start: function(tag, tagName, attrs) {
          var i;
          i = tagName.length - 1;
          name = (tagName[i] === ':' ? tagName.substr(0, i) : '').toLowerCase();
          from = attrs.from;
          importName = attrs["import"] && attrs["import"].toLowerCase();
          if (name === 'all' && importName) {
            throw new Error("Can't specify import attribute with All in " + path);
          }
        },
        chars: function(text, literal) {
          var _alias;
          if (!(get === 'all' || get === name)) {
            return;
          }
          got = true;
          if (from) {
            if (!onlyWhitespace.test(text)) {
              throw new Error("Template import '" + name + "' in " + path + " can't contain content");
            }
            if (importName) {
              _alias = name;
              name = importName;
            }
            return loadTemplates(join(dirname(path), from), null, name, _alias, templates, callback);
          }
          if (!(name && literal)) {
            if (onlyWhitespace.test(text)) {
              return;
            }
            throw new Error("Can't read template in " + path + " near the text: " + text);
          }
          return templates[alias || name] = trim(text);
        }
      });
      if (!got) {
        throw new Error("Can't find template '" + get + "' in " + path);
      }
      if (!--callback.count) {
        return callback(templates);
      }
    });
  });
};
module.exports = {
  isProduction: isProduction = process.env.NODE_ENV === 'production',
  css: function(root, clientName, callback) {
    var css;
    if (isProduction && (css = cssCache[clientName])) {
      return callback(css);
    }
    return findPath(root, clientName, 'styles', '.styl', function(path) {
      if (!path) {
        return callback(cssCache[path] = '');
      }
      return fs.readFile(path, 'utf8', function(err, styl) {
        if (err) {
          return callback(cssCache[path] = '');
        }
        return stylus(styl).use(nib()).set('filename', path).set('compress', true).render(function(err, css) {
          if (err) {
            throw err;
          }
          css = trim(css);
          return callback(cssCache[path] = css);
        });
      });
    });
  },
  views: views = function(view, root, clientName, callback) {
    return loadTemplates(root, clientName, 'all', null, {}, function(templates) {
      var name, text;
      for (name in templates) {
        text = templates[name];
        view.make(name, text);
        view._templates[name] = text;
      }
      return callback();
    });
  },
  js: function(view, parentFilename, options, callback) {
    var base, bundle, clientName, finish, js, minify, parentDir, root, staticDir, staticPath, staticRoot;
    if (!parentFilename) {
      return callback({});
    }
    root = parentDir = dirname(parentFilename);
    if ((base = basename(parentFilename, '.js')) === 'index') {
      base = basename(parentDir);
      root = dirname(dirname(parentDir));
    } else if (basename(parentDir) === 'lib') {
      root = dirname(parentDir);
    }
    clientName = options.name || base;
    staticRoot = options.staticRoot || join(root, 'public');
    staticDir = options.staticDir || 'gen';
    staticPath = join(staticRoot, staticDir);
    finish = function(js) {
      return views(view, root, clientName, function() {
        var filePath, filename, jsFile;
        js = js.replace('"$$templates$$"', JSON.stringify(view._templates || {}));
        filename = crypto.createHash('md5').update(js).digest('base64') + '.js';
        filename = filename.replace(/[\/\+=]/g, function(match) {
          switch (match) {
            case '/':
              return '-';
            case '+':
              return '_';
            case '=':
              return '';
          }
        });
        jsFile = join('/', staticDir, filename);
        filePath = join(staticPath, filename);
        return fs.writeFile(filePath, js, function() {
          return callback({
            root: root,
            clientName: clientName,
            jsFile: jsFile,
            require: basename(parentFilename)
          });
        });
      });
    };
    minify = 'minify' in options ? options.minify : isProduction;
    bundle = isProduction || !(js = jsCache[parentFilename]) ? function() {
      return fs.readFile(join(parentDir, 'inline.js'), 'utf8', function(err, inline) {
        racer.js({
          minify: minify,
          require: parentFilename
        }, function(js) {
          return finish(jsCache[parentFilename] = js);
        });
        if (err) {
          return;
        }
        return view.inline("function(){" + inline + "}");
      });
    } : function() {
      return finish(js);
    };
    return exists(staticPath, function(value) {
      if (value) {
        return bundle();
      }
      return exists(staticRoot, function(value) {
        if (value) {
          return fs.mkdir(staticPath, 0777, function(err) {
            return bundle();
          });
        }
        return fs.mkdir(staticRoot, 0777, function(err) {
          return fs.mkdir(staticPath, 0777, function(err) {
            return bundle();
          });
        });
      });
    });
  }
};

==> derby-0.1.3.tgz: package/lib/loader.js <==
var basename, crypto, cssCache, dirname, exists, findPath, fs, isProduction, join, jsCache, loadTemplates, nib, onlyWhitespace, parseHtml, racer, stylus, trim, views, _ref;

_ref = require('path'), dirname = _ref.dirname, basename = _ref.basename, join = _ref.join, exists = _ref.exists;

fs = require('fs');

crypto = require('crypto');

stylus = require('stylus');

nib = require('nib');

racer = require('racer');

trim = require('./View').trim;

parseHtml = require('./html').parse;

cssCache = {};

jsCache = {};

onlyWhitespace = /^[\s\n]*$/;

findPath = function(root, name, dir, extension, callback) {
  var path;
  if (name) root = join(root, dir, name);
  path = root + extension;
  return exists(path, function(value) {
    if (value) return callback(path);
    path = join(root, 'index' + extension);
    return exists(path, function(value) {
      return callback(value ? path : null);
    });
  });
};

loadTemplates = function(root, fileName, get, alias, templates, callback) {
  callback.count = (callback.count || 0) + 1;
  return findPath(root, fileName, 'views', '.html', function(path) {
    var got;
    if (!path) {
      if (fileName) {
        return callback({});
      } else {
        throw new Error("Can't find " + root + "/views/" + fileName);
      }
    }
    got = false;
    return fs.readFile(path, 'utf8', function(err, template) {
      var from, importName, name;
      if (err) return callback(err);
      name = '';
      from = '';
      importName = '';
      parseHtml(template, {
        start: function(tag, tagName, attrs) {
          var i;
          i = tagName.length - 1;
          name = (tagName[i] === ':' ? tagName.substr(0, i) : '').toLowerCase();
          from = attrs.from;
          importName = attrs["import"] && attrs["import"].toLowerCase();
          if (name === 'all' && importName) {
            throw new Error("Can't specify import attribute with All in " + path);
          }
        },
        chars: function(text, literal) {
          var _alias;
          if (!(get === 'all' || get === name)) return;
          got = true;
          if (from) {
            if (!onlyWhitespace.test(text)) {
              throw new Error("Template import '" + name + "' in " + path + " can't contain content");
            }
            if (importName) {
              _alias = name;
              name = importName;
            }
            return loadTemplates(join(dirname(path), from), null, name, _alias, templates, callback);
          }
          if (!(name && literal)) {
            if (onlyWhitespace.test(text)) return;
            throw new Error("Can't read template in " + path + " near the text: " + text);
          }
          return templates[alias || name] = trim(text);
        }
      });
      if (!got) throw new Error("Can't find template '" + get + "' in " + path);
      if (!--callback.count) return callback(templates);
    });
  });
};

module.exports = {
  isProduction: isProduction = process.env.NODE_ENV === 'production',
  css: function(root, clientName, callback) {
    var css;
    if (isProduction && (css = cssCache[clientName])) return callback(css);
    return findPath(root, clientName, 'styles', '.styl', function(path) {
      if (!path) return callback(cssCache[path] = '');
      return fs.readFile(path, 'utf8', function(err, styl) {
        if (err) return callback(cssCache[path] = '');
        return stylus(styl).use(nib()).set('filename', path).set('compress', true).render(function(err, css) {
          if (err) throw err;
          css = trim(css);
          return callback(cssCache[path] = css);
        });
      });
    });
  },
  views: views = function(view, root, clientName, callback) {
    return loadTemplates(root, clientName, 'all', null, {}, function(templates) {
      var name, text;
      view._templates || (view._templates = {});
      for (name in templates) {
        text = templates[name];
        view.make(name, text);
        view._templates[name] = text;
      }
      return callback();
    });
  },
  js: function(view, parentFilename, options, callback) {
    var base, bundle, clientName, finish, js, minify, parentDir, root, staticDir, staticPath, staticRoot;
    if (!parentFilename) return callback({});
    root = parentDir = dirname(parentFilename);
    if ((base = basename(parentFilename, '.js')) === 'index') {
      base = basename(parentDir);
      root = dirname(dirname(parentDir));
    } else if (basename(parentDir) === 'lib') {
      root = dirname(parentDir);
    }
    clientName = options.name || base;
    staticRoot = options.staticRoot || join(root, 'public');
    staticDir = options.staticDir || 'gen';
    staticPath = join(staticRoot, staticDir);
    finish = function(js) {
      return views(view, root, clientName, function() {
        var filePath, filename, jsFile;
        js = js.replace('"$$templates$$"', JSON.stringify(view._templates || {}));
        filename = crypto.createHash('md5').update(js).digest('base64') + '.js';
        filename = filename.replace(/[\/\+=]/g, function(match) {
          switch (match) {
            case '/':
              return '-';
            case '+':
              return '_';
            case '=':
              return '';
          }
        });
        jsFile = join('/', staticDir, filename);
        filePath = join(staticPath, filename);
        return fs.writeFile(filePath, js, function() {
          return callback({
            root: root,
            clientName: clientName,
            jsFile: jsFile,
            require: basename(parentFilename)
          });
        });
      });
    };
    minify = 'minify' in options ? options.minify : isProduction;
    bundle = isProduction || !(js = jsCache[parentFilename]) ? function() {
      return fs.readFile(join(parentDir, 'inline.js'), 'utf8', function(err, inline) {
        racer.js({
          minify: minify,
          require: parentFilename
        }, function(js) {
          return finish(jsCache[parentFilename] = js);
        });
        if (err) return;
        return view.inline("function(){" + inline + "}");
      });
    } : function() {
      return finish(js);
    };
    return exists(staticPath, function(value) {
      if (value) return bundle();
      return exists(staticRoot, function(value) {
        if (value) {
          return fs.mkdir(staticPath, 0777, function(err) {
            return bundle();
          });
        }
        return fs.mkdir(staticRoot, 0777, function(err) {
          return fs.mkdir(staticPath, 0777, function(err) {
            return bundle();
          });
        });
      });
    });
  }
};
